using Jypeli;

namespace TippuKivet
{
    public class TippuKivet : PhysicsGame
    {
        private readonly Vector nopeusOikea = new Vector(800, 0);
        private readonly Vector nopeusVasen = new Vector(-800, 0);

        private PhysicsObject pelaaja;
        private IntMeter pistelaskuri;
        private Timer ajastinTimantit;
        private Timer ajastinKivet;
        private int ohimenneetTimantit;

        public override void Begin()
        {
            SetWindowSize(1024, 768, false);

            ajastinTimantit = Timer.CreateAndStart(1.5, LuoTimantti);
            ajastinKivet = Timer.CreateAndStart(3.0, LuoKivi);

            pistelaskuri = new IntMeter(0);
            LuoPisteNaytto();

            Gravity = new Vector(0, -700);
            pelaaja = LuoPelaaja(0);
            LuoKenttä();
            AsetaOhjaimet();
        }

        private void LuoKenttä()
        {
            PhysicsObject alalaatta = Level.CreateBottomBorder();
            alalaatta.IsVisible = false;
            AddCollisionHandler(alalaatta, "Kivi", KiviAlalaattaCollide);
            AddCollisionHandler(alalaatta, "Timantti", TimanttiAlalaattaCollide);
            
            Level.Background.Color = Color.Black;
            Camera.ZoomToLevel();
        }

        private PhysicsObject LuoPelaaja(double x)
        {
            PhysicsObject pelaaja = LuoObjekti(350, 100, x, -400, Color.Red, "Pelaaja");
            pelaaja.IgnoresCollisionResponse = true;
            pelaaja.IgnoresGravity = true;

            AddCollisionHandler(pelaaja, "Kivi", PelaajaKiviCollide);
            AddCollisionHandler(pelaaja, "Timantti", PelaajaTimanttiCollide);
            
            Add(pelaaja);
            return pelaaja;
        }

        private void LuoPisteNaytto()
        {
            Label pisteNaytto = new Label
            {
                Position = new Vector(-400, 200),
                TextColor = Color.Red,
                Color = Color.White,
                Size = new Vector(5000, 5000)
            };
            pisteNaytto.BindTo(pistelaskuri);
            Add(pisteNaytto);
        }

        private void LuoTimantti()
        {
            double x = RandomGen.NextDouble(Level.Left + 50, Level.Right - 50);
            PhysicsObject timantti = LuoObjekti(60, 60, x, Level.Top + 50, Color.Yellow, "Timantti");
            timantti.AngularVelocity = RandomGen.NextDouble(-5.0, 5.0);
            AddCollisionHandler(timantti, "Pelaaja", PelaajaTimanttiCollide);
            Add(timantti);
        }

        private void LuoKivi()
        {
            double x = RandomGen.NextDouble(Level.Left + 20, Level.Right - 20);
            double randomSize = RandomGen.NextDouble(30.0, 120.0);
            PhysicsObject kivi = LuoObjekti(randomSize, randomSize, x, Level.Top + 50, Color.White, "Kivi");
            kivi.Velocity = new Vector(0, -400);
            AddCollisionHandler(kivi, "Pelaaja", PelaajaKiviCollide);
            Add(kivi);
        }
        
        private PhysicsObject LuoObjekti(double width, double height, double x, double y, Color color, string tag)
        {
            return new PhysicsObject(width, height, Shape.Rectangle)
            {
                X = x,
                Y = y,
                Color = color,
                Tag = tag
            };
        }

        private void PelaajaTimanttiCollide(PhysicsObject pelaaja, PhysicsObject timantti)
        {
            pistelaskuri.Value += 1;
            timantti.Destroy();
            KutistaPelaaja();

            if (pistelaskuri.Value >= 10)
            {
                VoitaPeli();
            }
        }

        private void PelaajaKiviCollide(PhysicsObject pelaaja, PhysicsObject kivi)
        {
            kivi.Destroy();
            LopetaPeli("Kivi osui sinua päähän, Peli ohi!");
        }

        private void KiviAlalaattaCollide(PhysicsObject alalaatta, PhysicsObject kivi)
        {
            kivi.Destroy();
        }

        private void TimanttiAlalaattaCollide(PhysicsObject alalaatta, PhysicsObject timantti)
        {
            timantti.Destroy();
            ohimenneetTimantit++;

            if (ohimenneetTimantit >= 2)
            {
                LopetaPeli("Et ollut tarpeeksi tehokas!");
            }
        }

        private void KutistaPelaaja()
        {
            const double shrinkFactor = 0.9;
            const double minSize = 10.0;

            if (pelaaja.Width > minSize && pelaaja.Height > minSize)
            {
                pelaaja.Width *= shrinkFactor;
                pelaaja.Height *= shrinkFactor;
            }
        }

        private void VoitaPeli()
        {
            MessageDisplay.Add("Olet TippuKiviLuolan suvereeni Päällikkö!");
            ajastinTimantit.Stop();
            ajastinKivet.Stop();
            Timer.SingleShot(6, Exit);
        }

        private void LopetaPeli(string viesti)
        {
            MessageDisplay.Add(viesti);
            ajastinTimantit.Stop();
            ajastinKivet.Stop();
            Timer.SingleShot(7, Exit);
        }

        private void AsetaOhjaimet()
        {
            Keyboard.Listen(Key.A, ButtonState.Down, AsetaNopeus, "Liikuta pelaajaa vasemmalle", pelaaja, nopeusVasen);
            Keyboard.Listen(Key.A, ButtonState.Released, AsetaNopeus, null, pelaaja, Vector.Zero);
            Keyboard.Listen(Key.D, ButtonState.Down, AsetaNopeus, "Liikuta pelaajaa oikealle", pelaaja, nopeusOikea);
            Keyboard.Listen(Key.D, ButtonState.Released, AsetaNopeus, null, pelaaja, Vector.Zero);
            Keyboard.Listen(Key.Left, ButtonState.Down, AsetaNopeus, "Liikuta pelaajaa vasemmalle nuolinäppäimellä", pelaaja, nopeusVasen);
            Keyboard.Listen(Key.Left, ButtonState.Released, AsetaNopeus, null, pelaaja, Vector.Zero);
            Keyboard.Listen(Key.Right, ButtonState.Down, AsetaNopeus, "Liikuta pelaajaa oikealle nuolinäppäimellä", pelaaja, nopeusOikea);
            Keyboard.Listen(Key.Right, ButtonState.Released, AsetaNopeus, null, pelaaja, Vector.Zero);
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
        }

        private void AsetaNopeus(PhysicsObject pelaaja, Vector nopeus)
        {
            if ((pelaaja.X + pelaaja.Width / 2 >= Level.Right && nopeus.X > 0) ||
                (pelaaja.X - pelaaja.Width / 2 <= Level.Left && nopeus.X < 0))
            {
                pelaaja.Velocity = Vector.Zero;
            }
            else
            {
                pelaaja.Velocity = nopeus;
            }
        }


    }
}
